@@ -16,11 +16,11 @@
  *
  *
  * Mail Notification
- * Copyright (C) 2003-2007 Jean-Yves Lefort <jylefort@brutele.be>
+ * Copyright (C) 2003-2008 Jean-Yves Lefort <jylefort@brutele.be>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -33,7 +33,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "config.h"
 #include <stdio.h>		/* required by stdlib.h on Darwin */
 #include <stdlib.h>		/* required by sys/socket.h on Darwin */
 #include <stdarg.h>
@@ -48,7 +47,6 @@
 #include <errno.h>
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <eel/eel.h>
 #if WITH_SSL
 #include <openssl/err.h>
 #include <openssl/x509v3.h>
@@ -61,15 +59,9 @@
 #endif /* WITH_SASL */
 #include "mn-util.h"
 #include "mn-client-session.h"
-#include "mn-conf.h"
-#include "mn-stock.h"
-
-/*** cpp *********************************************************************/
 
 #define READ_BUFSIZE			2048
 
-/*** types *******************************************************************/
-
 struct _MNClientSession
 {
   const MNClientSessionState		*states;
@@ -94,8 +86,6 @@ struct _MNClientSession
 #endif /* WITH_SASL */
 };
 
-/*** variables ***************************************************************/
-
 #if WITH_SASL
 static sasl_callback_t sasl_callbacks[] = {
   { SASL_CB_USER, NULL, NULL },
@@ -110,134 +100,64 @@ static sasl_callback_t sasl_callbacks[] = {
 G_LOCK_DEFINE_STATIC(resolver);
 #endif
 
-/*** functions ***************************************************************/
-
-static struct addrinfo *mn_client_session_resolve (MNClientSession *session);
-static int mn_client_session_connect (MNClientSession *session, struct addrinfo *addrinfo);
-
-#if WITH_SSL
-static GSList *mn_client_session_ssl_get_certificate_servers (X509 *cert);
-static gboolean mn_client_session_ssl_check_server_name (const char *user_name,
-							 const char *cert_name);
-static gboolean mn_client_session_ssl_check_server_name_from_list (const char *user_name,
-								   const GSList *cert_names);
-static char *mn_client_session_ssl_get_verify_error (MNClientSession *session,
-						     X509 *cert);
-static gboolean mn_client_session_ssl_verify (MNClientSession *session);
-static gboolean mn_client_session_run_untrusted_dialog (const char *server,
-							const char *reason,
-							const char *cert_fingerprint);
-#endif
-
-static int mn_client_session_enter_state (MNClientSession *session, int id);
-static gboolean mn_client_session_handle_input (MNClientSession *session, const char *input);
-
-static void mn_client_session_prepare_input_buffer (MNClientSession *session);
-
-#if WITH_SASL
-static int mn_client_session_write_base64 (MNClientSession *session,
-					   const char *buf,
-					   unsigned int len);
-static gboolean mn_client_session_sasl_fill_interact (MNClientSession *session,
-						      sasl_interact_t *interact,
-						      const char *unknown_warning);
-static char *mn_client_session_sasl_get_ip_port (const struct sockaddr *addr);
-#endif /* WITH_SASL */
-
-/*** implementation **********************************************************/
-
-/**
- * mn_client_session_run:
- * @states: a %MN_CLIENT_SESSION_STATES_END-terminated array of
- *          %MNClientSessionState structures. One of the states must
- *          have the %MN_CLIENT_SESSION_INITIAL_STATE id.
- * @callbacks: a pointer to a %MNClientSessionCallbacks structure
- * @use_ssl: whether to establish a SSL/TLS connection or not
- * @server: the hostname, IPv4 address or IPv6 address to connect to
- * @port: the port to connect to
- * @private: an opaque pointer which will be passed to callbacks, or %NULL
- * @err: a location to report errors, or %NULL
- *
- * Runs the client session. After connecting to the server, the
- * %MN_CLIENT_SESSION_INITIAL_STATE state is entered.
- *
- * Return value: %TRUE on success, or %FALSE on failure (in such case
- * @err is set)
- **/
-gboolean
-mn_client_session_run (const MNClientSessionState *states,
-		       const MNClientSessionCallbacks *callbacks,
-#if WITH_SSL
-		       gboolean use_ssl,
-#endif
-		       const char *server,
-		       int port,
-		       MNClientSessionPrivate *private,
-		       GError **err)
+static int
+mn_connect (int fd, const struct sockaddr *addr, socklen_t addrlen)
 {
-  MNClientSession session;
-  struct addrinfo *addrinfo;
-  const char *line;
+  int status;
 
-  g_return_val_if_fail(states != NULL, FALSE);
-  g_return_val_if_fail(callbacks != NULL, FALSE);
-  g_return_val_if_fail(callbacks->response_new != NULL, FALSE);
-  g_return_val_if_fail(server != NULL, FALSE);
+ again:
+  status = connect(fd, addr, addrlen);
+  if (status < 0)
+    switch (errno)
+      {
+      case EINTR:
+	goto again;
 
-  memset(&session, 0, sizeof(session));
-  session.states = states;
-  session.callbacks = callbacks;
-  session.server = server;
-  session.port = port;
-  session.private = private;
+      case EISCONN:
+	return 0;
+      }
 
-  addrinfo = mn_client_session_resolve(&session);
-  if (! addrinfo)
-    goto end;
+  return status;
+}
 
-  session.s = mn_client_session_connect(&session, addrinfo);
-  freeaddrinfo(addrinfo);
-  if (session.s < 0)
-    goto end;
+static ssize_t
+mn_read (int fd, void *buf, size_t count)
+{
+  ssize_t bytes_read;
 
-#if WITH_SSL
-  if (use_ssl)
-    {
-      if (! mn_client_session_enable_ssl(&session))
-	goto end;
-    }
-#endif /* WITH_SSL */
+  do
+    bytes_read = read(fd, buf, count);
+  while (bytes_read < 0 && errno == EINTR);
 
-  mn_client_session_enter_state(&session, MN_CLIENT_SESSION_INITIAL_STATE);
+  return bytes_read;
+}
 
-  session.input_buffer = g_byte_array_new();
-  while ((line = mn_client_session_read_line(&session)))
-    if (! mn_client_session_handle_input(&session, line))
-      break;
-  g_byte_array_free(session.input_buffer, TRUE);
+static ssize_t
+mn_write (int fd, const void *buf, size_t count)
+{
+  ssize_t bytes_written;
 
- end:
-  if (session.s >= 0)
-    while (close(session.s) < 0 && errno == EINTR);
-#if WITH_SSL
-  if (session.ssl)
-    SSL_free(session.ssl);
-#endif /* WITH_SSL */
-#if WITH_SASL
-  if (session.sasl_conn)
-    sasl_dispose(&session.sasl_conn);
-#endif /* WITH_SASL */
-  if (session.error)
-    {
-      g_propagate_error(err, session.error);
-      return FALSE;
-    }
-  else
-    return TRUE;
+  do
+    bytes_written = write(fd, buf, count);
+  while (bytes_written < 0 && errno == EINTR);
+
+  return bytes_written;
+}
+
+static int
+mn_close (int fd)
+{
+  int status;
+
+  do
+    status = close(fd);
+  while (status < 0 && errno == EINTR);
+
+  return status;
 }
 
 static struct addrinfo *
-mn_client_session_resolve (MNClientSession *session)
+resolve (MNClientSession *session)
 {
   char *servname;
   struct addrinfo hints;
@@ -276,7 +196,7 @@ mn_client_session_resolve (MNClientSession *session)
 }
 
 static int
-mn_client_session_connect (MNClientSession *session, struct addrinfo *addrinfo)
+client_session_connect (MNClientSession *session, struct addrinfo *addrinfo)
 {
   struct addrinfo *a;
   int n;
@@ -337,10 +257,10 @@ mn_client_session_connect (MNClientSession *session, struct addrinfo *addrinfo)
 	}
 
       mn_client_session_notice(session, _("connecting to %s (%s) port %i"), session->server, ip, session->port);
-      if (connect(s, a->ai_addr, a->ai_addrlen) < 0)
+      if (mn_connect(s, a->ai_addr, a->ai_addrlen) < 0)
 	{
 	  mn_client_session_notice(session, _("unable to connect: %s"), g_strerror(errno));
-	  close(s);
+	  mn_close(s);
 	}
       else
 	{
@@ -362,67 +282,186 @@ mn_client_session_connect (MNClientSession *session, struct addrinfo *addrinfo)
   return -1;
 }
 
-#if WITH_SSL
-/**
- * mn_client_session_enable_ssl:
- * @session: a #MNClientSession
- *
- * Enables in-band SSL/TLS. Must not be used if the @use_ssl
- * mn_client_session_run() argument was %TRUE. If an error occurs,
- * mn_client_session_set_error() will be called on @session.
- *
- * Return value: %TRUE on success
- **/
-gboolean
-mn_client_session_enable_ssl (MNClientSession *session)
+static int
+enter_state (MNClientSession *session, int id)
 {
-  SSL_CTX *ctx;
-  GError *err = NULL;
+  int i;
 
-  g_return_val_if_fail(session != NULL, FALSE);
-  g_return_val_if_fail(session->ssl == NULL, FALSE);
+  g_return_val_if_fail(session != NULL, 0);
 
-  ctx = mn_ssl_init(&err);
-  if (! ctx)
-    {
-      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("unable to initialize the OpenSSL library: %s"), err->message);
-      g_error_free(err);
-      return FALSE;
-    }
+  for (i = 0; session->states[i].id; i++)
+    if (session->states[i].id == id)
+      {
+	session->state = &session->states[i];
+	return session->state->enter_cb
+	  ? session->state->enter_cb(session, session->private)
+	  : MN_CLIENT_SESSION_RESULT_CONTINUE;
+      }
 
-  session->ssl = SSL_new(ctx);
-  if (! session->ssl)
-    {
-      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("unable to create a SSL/TLS object: %s"), mn_ssl_get_error());
-      return FALSE;
-    }
-
-  if (! SSL_set_fd(session->ssl, session->s))
-    {
-      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("unable to set the SSL/TLS file descriptor: %s"), mn_ssl_get_error());
-      return FALSE;
-    }
-
-  if (SSL_connect(session->ssl) != 1)
-    {
-      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("unable to perform the SSL/TLS handshake: %s"), mn_ssl_get_error());
-      return FALSE;
-    }
-
-  if (! mn_client_session_ssl_verify(session))
-    {
-      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("untrusted server"));
-      return FALSE;
-    }
-
-  mn_client_session_notice(session, _("a SSL/TLS layer is now active (%s, %s %i-bit)"),
-			   SSL_get_version(session->ssl),
-			   SSL_get_cipher(session->ssl),
-			   SSL_get_cipher_bits(session->ssl, NULL));
-
-  return TRUE;
+  g_assert_not_reached();
+  return 0;
 }
 
+static gboolean
+handle_input (MNClientSession *session, const char *input)
+{
+  MNClientSessionResponse *response;
+  gboolean cont = TRUE;
+
+  g_return_val_if_fail(session != NULL, FALSE);
+  g_return_val_if_fail(input != NULL, FALSE);
+
+  response = session->callbacks->response_new(session, input, session->private);
+  if (response)
+    {
+      int result;
+
+      g_assert(session->state->handle_cb != NULL);
+      result = session->state->handle_cb(session, response, session->private);
+
+    loop:
+      switch (result)
+	{
+	case MN_CLIENT_SESSION_RESULT_CONTINUE:
+	  break;
+
+	case MN_CLIENT_SESSION_RESULT_BAD_RESPONSE_FOR_CONTEXT:
+	  {
+	    char *escaped;
+
+	    escaped = mn_utf8_escape(input);
+	    mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("response \"%s\" is not valid in current context"), escaped);
+	    g_free(escaped);
+
+	    cont = FALSE;
+	  }
+	  break;
+
+	case MN_CLIENT_SESSION_RESULT_DISCONNECT:
+	  cont = FALSE;
+	  break;
+
+	case 0:			/* assertion failed somewhere */
+	  g_assert_not_reached();
+	  break;
+
+	default:
+	  g_assert(result > 0);
+	  result = enter_state(session, result);
+	  goto loop;
+	}
+
+      if (session->callbacks->response_free)
+	session->callbacks->response_free(session, response, session->private);
+    }
+  else
+    {
+      char *escaped;
+
+      escaped = mn_utf8_escape(input);
+      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("unable to parse response \"%s\""), escaped);
+      g_free(escaped);
+
+      cont = FALSE;
+    }
+
+  return cont;
+}
+
+/**
+ * mn_client_session_run:
+ * @states: a %MN_CLIENT_SESSION_STATES_END-terminated array of
+ *          %MNClientSessionState structures. One of the states must
+ *          have the %MN_CLIENT_SESSION_INITIAL_STATE id.
+ * @callbacks: a pointer to a %MNClientSessionCallbacks structure
+ * @use_ssl: whether to establish a SSL/TLS connection or not
+ * @server: the hostname, IPv4 address or IPv6 address to connect to
+ * @port: the port to connect to
+ * @private: an opaque pointer which will be passed to callbacks, or %NULL
+ * @err: a location to report errors, or %NULL
+ *
+ * Runs the client session. After connecting to the server, the
+ * %MN_CLIENT_SESSION_INITIAL_STATE state is entered.
+ *
+ * Return value: %TRUE on success, or %FALSE on failure (in such case
+ * @err is set)
+ **/
+gboolean
+mn_client_session_run (const MNClientSessionState *states,
+		       const MNClientSessionCallbacks *callbacks,
+#if WITH_SSL
+		       gboolean use_ssl,
+#endif
+		       const char *server,
+		       int port,
+		       MNClientSessionPrivate *private,
+		       GError **err)
+{
+  MNClientSession session;
+  struct addrinfo *addrinfo;
+  const char *line;
+
+  g_return_val_if_fail(states != NULL, FALSE);
+  g_return_val_if_fail(callbacks != NULL, FALSE);
+  g_return_val_if_fail(callbacks->response_new != NULL, FALSE);
+#if WITH_SSL
+  g_return_val_if_fail(callbacks->ssl_trust_server != NULL, FALSE);
+#endif
+  g_return_val_if_fail(server != NULL, FALSE);
+
+  memset(&session, 0, sizeof(session));
+  session.states = states;
+  session.callbacks = callbacks;
+  session.server = server;
+  session.port = port;
+  session.private = private;
+
+  addrinfo = resolve(&session);
+  if (! addrinfo)
+    goto end;
+
+  session.s = client_session_connect(&session, addrinfo);
+  freeaddrinfo(addrinfo);
+  if (session.s < 0)
+    goto end;
+
+#if WITH_SSL
+  if (use_ssl)
+    {
+      if (! mn_client_session_enable_ssl(&session))
+	goto end;
+    }
+#endif /* WITH_SSL */
+
+  enter_state(&session, MN_CLIENT_SESSION_INITIAL_STATE);
+
+  session.input_buffer = g_byte_array_new();
+  while ((line = mn_client_session_read_line(&session)))
+    if (! handle_input(&session, line))
+      break;
+  g_byte_array_free(session.input_buffer, TRUE);
+
+ end:
+  if (session.s >= 0)
+    mn_close(session.s);
+#if WITH_SSL
+  if (session.ssl)
+    SSL_free(session.ssl);
+#endif /* WITH_SSL */
+#if WITH_SASL
+  if (session.sasl_conn)
+    sasl_dispose(&session.sasl_conn);
+#endif /* WITH_SASL */
+  if (session.error)
+    {
+      g_propagate_error(err, session.error);
+      return FALSE;
+    }
+  else
+    return TRUE;
+}
+
+#if WITH_SSL
 /**
  * mn_client_session_ssl_get_certificate_servers():
  * @cert: the server certificate
@@ -431,10 +470,10 @@ mn_client_session_enable_ssl (MNClientSession *session)
  * contained in @cert.
  *
  * Return value: a newly-allocated list of UTF-8 server names. When no
- * longer used, the list must be freed with eel_g_slist_free_deep().
+ * longer used, the list must be freed with mn_g_slist_free_deep().
  **/
 static GSList *
-mn_client_session_ssl_get_certificate_servers (X509 *cert)
+get_ssl_certificate_servers (X509 *cert)
 {
   GSList *servers = NULL;
   X509_NAME *subject;
@@ -509,8 +548,7 @@ mn_client_session_ssl_get_certificate_servers (X509 *cert)
 }
 
 static gboolean
-mn_client_session_ssl_check_server_name (const char *user_name,
-					 const char *cert_name)
+check_ssl_server_name (const char *user_name, const char *cert_name)
 {
   g_return_val_if_fail(user_name != NULL, FALSE);
   g_return_val_if_fail(cert_name != NULL, FALSE);
@@ -533,10 +571,9 @@ mn_client_session_ssl_check_server_name (const char *user_name,
 }
 
 static gboolean
-mn_client_session_ssl_check_server_name_from_list (const char *user_name,
-						   const GSList *cert_names)
+check_ssl_server_name_from_list (const char *user_name, GSList *cert_names)
 {
-  const GSList *l;
+  GSList *l;
 
   g_return_val_if_fail(user_name != NULL, FALSE);
 
@@ -544,7 +581,7 @@ mn_client_session_ssl_check_server_name_from_list (const char *user_name,
     {
       const char *cert_name = l->data;
 
-      if (mn_client_session_ssl_check_server_name(user_name, cert_name))
+      if (check_ssl_server_name(user_name, cert_name))
 	return TRUE;
     }
 
@@ -552,7 +589,7 @@ mn_client_session_ssl_check_server_name_from_list (const char *user_name,
 }
 
 static char *
-mn_client_session_ssl_get_verify_error (MNClientSession *session, X509 *cert)
+get_ssl_verify_error (MNClientSession *session, X509 *cert)
 {
   long verify_result;
   GSList *servers;
@@ -584,11 +621,11 @@ mn_client_session_ssl_get_verify_error (MNClientSession *session, X509 *cert)
    * 3501 11.1) and cannot hurt for POP3.
    */
 
-  servers = mn_client_session_ssl_get_certificate_servers(cert);
+  servers = get_ssl_certificate_servers(cert);
   if (! servers)
     return g_strdup(_("server name not found in certificate"));
 
-  if (! mn_client_session_ssl_check_server_name_from_list(session->server, servers))
+  if (! check_ssl_server_name_from_list(session->server, servers))
     {
       if (g_slist_length(servers) == 1)
 	error = g_strdup_printf(_("user-provided server name \"%s\" does not match certificate-provided server name \"%s\""),
@@ -604,9 +641,10 @@ mn_client_session_ssl_get_verify_error (MNClientSession *session, X509 *cert)
 	    {
 	      char *server = l->data;
 
-	      g_string_append_printf(servers_comma_list, "\"%s\"", server);
 	      if (l->next)
-		g_string_append(servers_comma_list, ", ");
+		g_string_append_printf(servers_comma_list, _("\"%s\", "), server);
+	      else
+		g_string_append_printf(servers_comma_list, _("\"%s\""), server);
 	    }
 
 	  error = g_strdup_printf(_("user-provided server name \"%s\" matches none of the certificate-provided server names %s"),
@@ -616,16 +654,15 @@ mn_client_session_ssl_get_verify_error (MNClientSession *session, X509 *cert)
 	}
     }
 
-  eel_g_slist_free_deep(servers);
+  mn_g_slist_free_deep(servers);
 
   return error;
 }
 
 static gboolean
-mn_client_session_ssl_verify (MNClientSession *session)
+verify_ssl_certificate (MNClientSession *session)
 {
   X509 *cert;
-  gboolean status = FALSE;
 
   g_return_val_if_fail(session != NULL, FALSE);
   g_return_val_if_fail(session->ssl != NULL, FALSE);
@@ -634,8 +671,9 @@ mn_client_session_ssl_verify (MNClientSession *session)
   if (cert)
     {
       char *error;
+      gboolean status = FALSE;
 
-      error = mn_client_session_ssl_get_verify_error(session, cert);
+      error = get_ssl_verify_error(session, cert);
       if (! error)
 	status = TRUE;
       else
@@ -645,7 +683,6 @@ mn_client_session_ssl_verify (MNClientSession *session)
 	  int md5len;
 	  int i;
 	  unsigned char *f;
-	  GSList *gconf_fingerprints;
 
 	  /* calculate the MD5 hash of the raw certificate */
 	  md5len = sizeof(md5sum);
@@ -653,193 +690,95 @@ mn_client_session_ssl_verify (MNClientSession *session)
 	  for (i = 0, f = fingerprint; i < 16; i++, f += 3)
 	    sprintf(f, "%.2x%c", md5sum[i], i != 15 ? ':' : '\0');
 
-	  gconf_fingerprints = eel_gconf_get_string_list(MN_CONF_TRUSTED_X509_CERTIFICATES);
-
-	  if (mn_g_str_slist_find(gconf_fingerprints, fingerprint) != NULL)
+	  if (session->callbacks->ssl_trust_server(session,
+						   session->server,
+						   session->port,
+						   fingerprint,
+						   error,
+						   session->private))
 	    status = TRUE;
-	  else
-	    {
-	      if (mn_client_session_run_untrusted_dialog(session->server, error, fingerprint))
-		{
-		  status = TRUE;
-		  gconf_fingerprints = g_slist_append(gconf_fingerprints, g_strdup(fingerprint));
-		  eel_gconf_set_string_list(MN_CONF_TRUSTED_X509_CERTIFICATES, gconf_fingerprints);
-		}
-	    }
 
-	  eel_g_slist_free_deep(gconf_fingerprints);
 	  g_free(error);
 	}
 
       X509_free(cert);
+
+      return status;
     }
   else
-    {
-      char *server;
-      GSList *gconf_servers = NULL;
-
-      server = g_strdup_printf("%s:%i", session->server, session->port);
-      gconf_servers = eel_gconf_get_string_list(MN_CONF_TRUSTED_SERVERS);
-
-      if (mn_g_str_slist_find(gconf_servers, server) != NULL)
-	status = TRUE;
-      else
-	{
-	  if (mn_client_session_run_untrusted_dialog(session->server, _("missing certificate"), NULL))
-	    {
-	      status = TRUE;
-	      gconf_servers = g_slist_append(gconf_servers, g_strdup(server));
-	      eel_gconf_set_string_list(MN_CONF_TRUSTED_SERVERS, gconf_servers);
-	    }
-	}
-
-      g_free(server);
-      eel_g_slist_free_deep(gconf_servers);
-    }
-
-  return status;
+    return session->callbacks->ssl_trust_server(session,
+						session->server,
+						session->port,
+						NULL,
+						NULL,
+						session->private);
 }
 
-static gboolean
-mn_client_session_run_untrusted_dialog (const char *server,
-					const char *reason,
-					const char *cert_fingerprint)
+/**
+ * mn_client_session_enable_ssl:
+ * @session: a #MNClientSession
+ *
+ * Enables in-band SSL/TLS. Must not be used if the @use_ssl
+ * mn_client_session_run() argument was %TRUE. If an error occurs,
+ * mn_client_session_set_error() will be called on @session.
+ *
+ * Return value: %TRUE on success
+ **/
+gboolean
+mn_client_session_enable_ssl (MNClientSession *session)
 {
-  GtkWidget *dialog;
-  GString *secondary;
-  gboolean status;
+  SSL_CTX *ctx;
+  GError *err = NULL;
+  int ret;
 
-  g_return_val_if_fail(server != NULL, FALSE);
-  g_return_val_if_fail(reason != NULL, FALSE);
+  g_return_val_if_fail(session != NULL, FALSE);
+  g_return_val_if_fail(session->ssl == NULL, FALSE);
 
-  secondary = g_string_new(NULL);
-  g_string_printf(secondary,
-		  _("Mail Notification was unable to trust \"%s\" "
-		    "(%s). It is possible that someone is "
-		    "intercepting your communication to obtain "
-		    "your confidential information.\n"
-		    "\n"
-		    "You should only connect to the server if you "
-		    "are certain you are connected to \"%s\". "
-		    "If you choose to connect to the server, this "
-		    "message will not be shown again."),
-		  server, reason, server);
-
-  if (cert_fingerprint)
+  ctx = mn_ssl_init(&err);
+  if (! ctx)
     {
-      g_string_append(secondary, "\n\n");
-      g_string_append_printf(secondary, _("Certificate fingerprint: %s."), cert_fingerprint);
+      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("unable to initialize the OpenSSL library: %s"), err->message);
+      g_error_free(err);
+      return FALSE;
     }
 
-  GDK_THREADS_ENTER();
+  session->ssl = SSL_new(ctx);
+  if (! session->ssl)
+    {
+      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("unable to create a SSL/TLS object: %s"), mn_ssl_get_error());
+      return FALSE;
+    }
 
-  dialog = mn_alert_dialog_new(NULL,
-			       GTK_MESSAGE_WARNING, 0,
-			       _("Connect to untrusted server?"),
-			       secondary->str);
-  g_string_free(secondary, TRUE);
+  if (! SSL_set_fd(session->ssl, session->s))
+    {
+      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("unable to set the SSL/TLS file descriptor: %s"), mn_ssl_get_error());
+      return FALSE;
+    }
 
-  gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-  gtk_dialog_add_button(GTK_DIALOG(dialog), MN_STOCK_CONNECT, GTK_RESPONSE_OK);
+  ret = SSL_connect(session->ssl);
+  if (ret != 1)
+    {
+      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("unable to perform the SSL/TLS handshake: %s"), mn_ssl_get_io_error(session->ssl, ret));
+      return FALSE;
+    }
 
-  status = mn_dialog_run_nonmodal(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK;
-  gtk_widget_destroy(dialog);
+  if (! verify_ssl_certificate(session))
+    {
+      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("untrusted server"));
+      return FALSE;
+    }
 
-  gdk_flush();
-  GDK_THREADS_LEAVE();
+  mn_client_session_notice(session, _("a SSL/TLS layer is now active (%s, %s %i-bit)"),
+			   SSL_get_version(session->ssl),
+			   SSL_get_cipher(session->ssl),
+			   SSL_get_cipher_bits(session->ssl, NULL));
 
-  return status;
+  return TRUE;
 }
 #endif /* WITH_SSL */
 
-static int
-mn_client_session_enter_state (MNClientSession *session, int id)
-{
-  int i;
-
-  g_return_val_if_fail(session != NULL, 0);
-
-  for (i = 0; session->states[i].id; i++)
-    if (session->states[i].id == id)
-      {
-	session->state = &session->states[i];
-	return session->state->enter_cb
-	  ? session->state->enter_cb(session, session->private)
-	  : MN_CLIENT_SESSION_RESULT_CONTINUE;
-      }
-
-  g_assert_not_reached();
-  return 0;
-}
-
-static gboolean
-mn_client_session_handle_input (MNClientSession *session, const char *input)
-{
-  MNClientSessionResponse *response;
-  gboolean cont = TRUE;
-
-  g_return_val_if_fail(session != NULL, FALSE);
-  g_return_val_if_fail(input != NULL, FALSE);
-
-  response = session->callbacks->response_new(session, input, session->private);
-  if (response)
-    {
-      int result;
-
-      g_assert(session->state->handle_cb != NULL);
-      result = session->state->handle_cb(session, response, session->private);
-
-    loop:
-      switch (result)
-	{
-	case MN_CLIENT_SESSION_RESULT_CONTINUE:
-	  break;
-
-	case MN_CLIENT_SESSION_RESULT_BAD_RESPONSE_FOR_CONTEXT:
-	  {
-	    char *escaped;
-
-	    escaped = mn_utf8_escape(input);
-	    mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("response \"%s\" is not valid in current context"), escaped);
-	    g_free(escaped);
-
-	    cont = FALSE;
-	  }
-	  break;
-
-	case MN_CLIENT_SESSION_RESULT_DISCONNECT:
-	  cont = FALSE;
-	  break;
-
-	case 0:			/* assertion failed somewhere */
-	  g_assert_not_reached();
-	  break;
-
-	default:
-	  g_assert(result > 0);
-	  result = mn_client_session_enter_state(session, result);
-	  goto loop;
-	}
-
-      if (session->callbacks->response_free)
-	session->callbacks->response_free(session, response, session->private);
-    }
-  else
-    {
-      char *escaped;
-
-      escaped = mn_utf8_escape(input);
-      mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_OTHER, _("unable to parse response \"%s\""), escaped);
-      g_free(escaped);
-
-      cont = FALSE;
-    }
-
-  return cont;
-}
-
 static void
-mn_client_session_prepare_input_buffer (MNClientSession *session)
+prepare_input_buffer (MNClientSession *session)
 {
   g_return_if_fail(session != NULL);
 
@@ -868,9 +807,7 @@ mn_client_session_fill_input_buffer (MNClientSession *session)
     bytes_read = SSL_read(session->ssl, buf, sizeof(buf));
   else
 #endif /* WITH_SSL */
-    do
-      bytes_read = read(session->s, buf, sizeof(buf));
-    while (bytes_read < 0 && errno == EINTR);
+    bytes_read = mn_read(session->s, buf, sizeof(buf));
 
   if (session->callbacks->post_read)
     session->callbacks->post_read(session, session->private);
@@ -879,7 +816,7 @@ mn_client_session_fill_input_buffer (MNClientSession *session)
     {
 #if WITH_SSL
       if (session->ssl)
-	mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_CONNECTION_LOST, _("unable to read from server: %s"), mn_ssl_get_error());
+	mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_CONNECTION_LOST, _("unable to read from server: %s"), mn_ssl_get_io_error(session->ssl, bytes_read));
       else
 #endif /* WITH_SSL */
 	{
@@ -933,7 +870,7 @@ mn_client_session_read (MNClientSession *session, unsigned int nbytes)
   g_return_val_if_fail(session->input_buffer != NULL, FALSE);
   g_return_val_if_fail(nbytes >= 0, FALSE);
 
-  mn_client_session_prepare_input_buffer(session);
+  prepare_input_buffer(session);
 
   while (session->input_buffer->len < nbytes)
     if (! mn_client_session_fill_input_buffer(session))
@@ -969,7 +906,7 @@ mn_client_session_read_line (MNClientSession *session)
   g_return_val_if_fail(session != NULL, NULL);
   g_return_val_if_fail(session->input_buffer != NULL, NULL);
 
-  mn_client_session_prepare_input_buffer(session);
+  prepare_input_buffer(session);
 
   while (! (session->input_buffer->data
 	    && (terminator = g_strstr_len(session->input_buffer->data,
@@ -1006,7 +943,6 @@ mn_client_session_write (MNClientSession *session,
 			 const char *format,
 			 ...)
 {
-  va_list args;
   char *str;
   char *full;
   unsigned int len;
@@ -1017,13 +953,13 @@ mn_client_session_write (MNClientSession *session,
   g_return_val_if_fail(session != NULL, 0);
   g_return_val_if_fail(format != NULL, 0);
 
-  va_start(args, format);
-  str = g_strdup_vprintf(format, args);
-  va_end(args);
+  MN_STRDUP_VPRINTF(str, format);
 
   mn_client_session_notice(session, "> %s", str);
+
   full = g_strconcat(str, "\r\n", NULL);
   g_free(str);
+
   len = strlen(full);
 
 #if WITH_SASL
@@ -1064,15 +1000,13 @@ mn_client_session_write (MNClientSession *session,
     bytes_written = SSL_write(session->ssl, array->data, array->len);
   else
 #endif /* WITH_SSL */
-    do
-      bytes_written = write(session->s, array->data, array->len);
-    while (bytes_written < 0 && errno == EINTR);
+    bytes_written = mn_write(session->s, array->data, array->len);
 
   if (bytes_written <= 0)
     {
 #if WITH_SSL
       if (session->ssl)
-	result = mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_CONNECTION_LOST, _("unable to write to server: %s"), mn_ssl_get_error());
+	result = mn_client_session_set_error(session, MN_CLIENT_SESSION_ERROR_CONNECTION_LOST, _("unable to write to server: %s"), mn_ssl_get_io_error(session->ssl, bytes_written));
       else
 #endif /* WITH_SSL */
 	{
@@ -1094,9 +1028,7 @@ mn_client_session_write (MNClientSession *session,
 
 #if WITH_SASL
 static int
-mn_client_session_write_base64 (MNClientSession *session,
-				const char *buf,
-				unsigned int len)
+write_base64 (MNClientSession *session, const char *buf, unsigned int len)
 {
   char buf64[len * 2];		/* Base64 is 33% larger than the data it encodes */
   unsigned int outlen;
@@ -1118,9 +1050,9 @@ mn_client_session_write_base64 (MNClientSession *session,
 }
 
 static gboolean
-mn_client_session_sasl_fill_interact (MNClientSession *session,
-				      sasl_interact_t *interact,
-				      const char *unknown_warning)
+fill_sasl_interact (MNClientSession *session,
+		    sasl_interact_t *interact,
+		    const char *unknown_warning)
 {
   sasl_interact_t *i;
   gboolean need_username = FALSE;
@@ -1144,7 +1076,7 @@ mn_client_session_sasl_fill_interact (MNClientSession *session,
 	break;
 
       default:
-	mn_client_session_warning(session, unknown_warning);
+	mn_client_session_warning(session, "%s", unknown_warning);
 	return FALSE;
       };
 
@@ -1187,7 +1119,7 @@ mn_client_session_sasl_fill_interact (MNClientSession *session,
 }
 
 static char *
-mn_client_session_sasl_get_ip_port (const struct sockaddr *addr)
+get_sasl_ip_port (const struct sockaddr *addr)
 {
 #if WITH_IPV6
   char buf[INET6_ADDRSTRLEN];
@@ -1295,13 +1227,13 @@ mn_client_session_sasl_authentication_start (MNClientSession *session,
 
   namelen = sizeof(name);
   if (getsockname(session->s, &name, &namelen) >= 0)
-    local_ip_port = mn_client_session_sasl_get_ip_port(&name);
+    local_ip_port = get_sasl_ip_port(&name);
   else
     mn_client_session_warning(session, _("unable to retrieve local address of socket: %s"), g_strerror(errno));
 
   namelen = sizeof(name);
   if (getpeername(session->s, &name, &namelen) >= 0)
-    remote_ip_port = mn_client_session_sasl_get_ip_port(&name);
+    remote_ip_port = get_sasl_ip_port(&name);
   else
     mn_client_session_warning(session, _("unable to retrieve remote address of socket: %s"), g_strerror(errno));
 
@@ -1361,7 +1293,7 @@ mn_client_session_sasl_authentication_start (MNClientSession *session,
 
 	  if (result == SASL_INTERACT)
 	    {
-	      if (! mn_client_session_sasl_fill_interact(session, interact, _("unable to start SASL authentication: SASL asked for something we did not know")))
+	      if (! fill_sasl_interact(session, interact, _("unable to start SASL authentication: SASL asked for something we did not know")))
 		break;
 	    }
 	}
@@ -1431,7 +1363,7 @@ mn_client_session_sasl_authentication_step (MNClientSession *session,
 
 	    if (result == SASL_INTERACT)
 	      {
-		if (! mn_client_session_sasl_fill_interact(session, interact, _("SASL asked for something we did not know, aborting SASL authentication")))
+		if (! fill_sasl_interact(session, interact, _("SASL asked for something we did not know, aborting SASL authentication")))
 		  break;
 	      }
 	  }
@@ -1441,7 +1373,7 @@ mn_client_session_sasl_authentication_step (MNClientSession *session,
 	  {
 	  case SASL_OK:
 	  case SASL_CONTINUE:
-	    return mn_client_session_write_base64(session, clientout, clientoutlen);
+	    return write_base64(session, clientout, clientoutlen);
 
 	  case SASL_INTERACT:
 	    /* could not fill interaction, abort */
@@ -1517,7 +1449,8 @@ mn_client_session_sasl_authentication_done (MNClientSession *session)
  * will not leak the object away. However, in some situations (eg. if
  * SASL authentication fails but the session continues nevertheless)
  * it might be desirable to get rid of the object, in order to free
- * memory for the rest of the session duration.
+ * resources that would otherwise remain in use for the rest of the
+ * session.
  **/
 void
 mn_client_session_sasl_dispose (MNClientSession *session)
@@ -1570,13 +1503,9 @@ mn_client_session_notice (MNClientSession *session,
 
   if (session->callbacks->notice)
     {
-      va_list args;
       char *message;
 
-      va_start(args, format);
-      message = g_strdup_vprintf(format, args);
-      va_end(args);
-
+      MN_STRDUP_VPRINTF(message, format);
       session->callbacks->notice(session, message, session->private);
       g_free(message);
     }
@@ -1601,13 +1530,9 @@ mn_client_session_warning (MNClientSession *session,
 
   if (session->callbacks->warning)
     {
-      va_list args;
       char *message;
 
-      va_start(args, format);
-      message = g_strdup_vprintf(format, args);
-      va_end(args);
-
+      MN_STRDUP_VPRINTF(message, format);
       session->callbacks->warning(session, message, session->private);
       g_free(message);
     }
@@ -1636,13 +1561,9 @@ mn_client_session_set_error (MNClientSession *session,
 
   if (! session->error)
     {
-      va_list args;
       char *message;
 
-      va_start(args, format);
-      message = g_strdup_vprintf(format, args);
-      va_end(args);
-
+      MN_STRDUP_VPRINTF(message, format);
       session->error = g_error_new_literal(MN_CLIENT_SESSION_ERROR, code, message);
       g_free(message);
     }
@@ -1658,7 +1579,7 @@ mn_client_session_set_error_from_response (MNClientSession *session,
   g_return_val_if_fail(session != NULL, 0);
 
   return response
-    ? mn_client_session_set_error(session, code, "\"%s\"", response)
+    ? mn_client_session_set_error(session, code, _("\"%s\""), response)
     : mn_client_session_set_error(session, code, _("unknown server error"));
 }
 
