@@ -1,10 +1,10 @@
 /*
  * Mail Notification
- * Copyright (C) 2003-2007 Jean-Yves Lefort <jylefort@brutele.be>
+ * Copyright (C) 2003-2008 Jean-Yves Lefort <jylefort@brutele.be>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -20,7 +20,6 @@
 #ifndef _MN_CLIENT_SESSION_H
 #define _MN_CLIENT_SESSION_H
 
-#include "config.h"
 #include <glib.h>
 #if WITH_SASL
 #include <sasl/sasl.h>
@@ -173,6 +172,24 @@ typedef struct
 					 const char		**username,
 					 const char		**password);
 #endif /* WITH_SASL */
+
+#if WITH_SSL
+  /*
+   * ssl_trust_server (required if SSL support was compiled in):
+   *
+   * Decides whether to trust the given server after SSL verification
+   * failed. If the server provided a certificate, the @fingerprint
+   * and @verify_error arguments will be set.
+   *
+   * Return value: must return %TRUE if the given server should be trusted.
+   */
+  gboolean  (*ssl_trust_server)		(MNClientSession	*session,
+					 const char		*server,
+					 int			port,
+					 const char		*fingerprint,
+					 const char		*verify_error,
+					 MNClientSessionPrivate	*priv);
+#endif /* WITH_SSL */
 } MNClientSessionCallbacks;
 
 gboolean mn_client_session_run (const MNClientSessionState *states,
@@ -217,7 +234,7 @@ void mn_client_session_notice (MNClientSession *session,
 			       ...) G_GNUC_PRINTF(2, 3);
 void mn_client_session_warning (MNClientSession *session,
 				const char *format,
-				...);
+				...) G_GNUC_PRINTF(2, 3);
 
 int mn_client_session_set_error (MNClientSession *session,
 				 int code,
