@@ -1,10 +1,10 @@
 /*
  * Mail Notification
- * Copyright (C) 2003-2007 Jean-Yves Lefort <jylefort@brutele.be>
+ * Copyright (C) 2003-2008 Jean-Yves Lefort <jylefort@brutele.be>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,21 +17,16 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "config.h"
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <sasl/sasl.h>
 
-/*** variables ***************************************************************/
-
 static gboolean attempted = FALSE;
 static gboolean initialized = FALSE;
 static char *init_error = NULL;
 
 G_LOCK_DEFINE_STATIC(init);
 
-/*** implementation **********************************************************/
-
 gboolean
 mn_sasl_init (GError **err)
 {
