@@ -1,10 +1,10 @@
 /*
  * Mail Notification
- * Copyright (C) 2003-2007 Jean-Yves Lefort <jylefort@brutele.be>
+ * Copyright (C) 2003-2008 Jean-Yves Lefort <jylefort@brutele.be>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,14 +17,11 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "config.h"
 #include <string.h>
 #include <glib/gi18n.h>
 #include "mn-xml.h"
 #include "mn-util.h"
 
-/*** implementation **********************************************************/
-
 void
 mn_xml_import_properties (GObject *object, xmlNode *node)
 {
@@ -37,7 +34,7 @@ mn_xml_import_properties (GObject *object, xmlNode *node)
 
   properties = g_object_class_list_properties(G_OBJECT_GET_CLASS(object), &n_properties);
   for (i = 0; i < n_properties; i++)
-    if ((properties[i]->flags & MN_XML_PARAM_IMPORT_EXPORT) != 0)
+    if ((properties[i]->flags & MN_XML_PARAM_IMPORT) != 0)
       {
 	char *content;
 
@@ -73,7 +70,7 @@ mn_xml_export_properties (GObject *object, xmlNode *node)
 
   properties = g_object_class_list_properties(G_OBJECT_GET_CLASS(object), &n_properties);
   for (i = 0; i < n_properties; i++)
-    if ((properties[i]->flags & MN_XML_PARAM_IMPORT_EXPORT) != 0)
+    if ((properties[i]->flags & MN_XML_PARAM_EXPORT) != 0)
       {
 	GValue value = { 0, };
 	gboolean is_default;
