@@ -1,10 +1,10 @@
 /*
  * Mail Notification
- * Copyright (C) 2003-2007 Jean-Yves Lefort <jylefort@brutele.be>
+ * Copyright (C) 2003-2008 Jean-Yves Lefort <jylefort@brutele.be>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,14 +25,20 @@
 
 typedef enum
 {
-  /* import/export this property */
-  MN_XML_PARAM_IMPORT_EXPORT	= 1 << (G_PARAM_USER_SHIFT + 0),
+  /* import this property in mn_xml_import_properties() */
+  MN_XML_PARAM_IMPORT		= 1 << (G_PARAM_USER_SHIFT + 0),
+
+  /* export this property in mn_xml_export_properties() */
+  MN_XML_PARAM_EXPORT		= 1 << (G_PARAM_USER_SHIFT + 1),
+
+  /* import and export this property */
+  MN_XML_PARAM_IMPORT_EXPORT	= MN_XML_PARAM_IMPORT | MN_XML_PARAM_EXPORT,
 
   /* ignore the case of a string property when checking if it has the default value */
-  MN_XML_PARAM_IGNORE_CASE	= 1 << (G_PARAM_USER_SHIFT + 1)
+  MN_XML_PARAM_IGNORE_CASE	= 1 << (G_PARAM_USER_SHIFT + 2)
 } MNXMLParamFlags;
 
-#define MN_XML_PARAM_USER_SHIFT		(G_PARAM_USER_SHIFT + 2)
+#define MN_XML_PARAM_USER_SHIFT		(G_PARAM_USER_SHIFT + 3)
 
 void mn_xml_import_properties (GObject *object, xmlNode *node);
 void mn_xml_export_properties (GObject *object, xmlNode *node);
