@@ -1,30 +1,26 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
-
-   nautilus-cell-renderer-pixbuf-emblem.c: cell renderer which can render
-   an emblem on top of a pixbuf (for use in FMListView and FMTreeView)
-
-   Copyright (C) 2003 Juerg Billeter
-
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   This is based on GtkCellRendererPixbuf written by
-   Jonathan Blandford <jrb@redhat.com>
-
-   Author: Juerg Billeter <j@bitron.ch>
-*/
+ *
+ * nautilus-cell-renderer-pixbuf-emblem.c: cell renderer which can render
+ * an emblem on top of a pixbuf (for use in FMListView and FMTreeView)
+ * Copyright (C) 2003 Juerg Billeter
+ *
+ * Mail Notification
+ * Copyright (C) 2003-2008 Jean-Yves Lefort <jylefort@brutele.be>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include "nautilus-cell-renderer-pixbuf-emblem.h"
 
@@ -398,7 +394,7 @@ nautilus_cell_renderer_pixbuf_emblem_get_size (GtkCellRenderer *cell,
 	if (cell_area && pixbuf_width > 0 && pixbuf_height > 0) {
 		if (x_offset) {
 			*x_offset = (((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) ?
-				1.0 - cell->xalign : cell->xalign) * 
+				1.0 - cell->xalign : cell->xalign) *
 				(cell_area->width - calc_width - 2 * cell->xpad));
 			*x_offset = MAX (*x_offset, 0) + cell->xpad;
 		}
